#!/usr/bin/python
#
# Copyright (C) 2010, Nokia <jean-luc.lamadon@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"Constants describing Tracker D-Bus services"

import os

TRACKER_BUSNAME = 'org.freedesktop.Tracker1'
TRACKER_OBJ_PATH = '/org/freedesktop/Tracker1/Resources'
RESOURCES_IFACE = "org.freedesktop.Tracker1.Resources"

MINERFS_BUSNAME = "org.freedesktop.Tracker1.Miner.Files"
MINERFS_OBJ_PATH = "/org/freedesktop/Tracker1/Miner/Files"
MINER_IFACE = "org.freedesktop.Tracker1.Miner"
MINERFS_INDEX_OBJ_PATH = "/org/freedesktop/Tracker1/Miner/Files/Index"
MINER_INDEX_IFACE = "org.freedesktop.Tracker1.Miner.Index"

TRACKER_BACKUP_OBJ_PATH = "/org/freedesktop/Tracker1/Backup"                                            
BACKUP_IFACE = "org.freedesktop.Tracker1.Backup"

TRACKER_STATS_OBJ_PATH = "/org/freedesktop/Tracker1/Statistics"
STATS_IFACE = "org.freedesktop.Tracker1.Statistics"

TRACKER_STATUS_OBJ_PATH = "/org/freedesktop/Tracker1/Status"
STATUS_IFACE = "org.freedesktop.Tracker1.Status"

TRACKER_EXTRACT_BUSNAME = "org.freedesktop.Tracker1.Miner.Extract"
TRACKER_EXTRACT_OBJ_PATH = "/org/freedesktop/Tracker1/Miner/Extract"

WRITEBACK_BUSNAME = "org.freedesktop.Tracker1.Writeback"


DCONF_MINER_SCHEMA = "org.freedesktop.Tracker.Miner.Files"

def expandvars (variable):
    # Note: the order matters!
    result = variable
    for var, value in [("${datarootdir}", RAW_DATAROOT_DIR),
                       ("${exec_prefix}", RAW_EXEC_PREFIX),
                       ("${prefix}", PREFIX)]:
        result = result.replace (var, value)


    return result



PREFIX = "/home/carlos/Build/gnome"
#
# This raw variables are set by autotools without translating vars:
#   E.G. bindir='${exec_prefix}/bin
#
# So we do the translation by hand in the expandvars function
#
RAW_EXEC_PREFIX = "${prefix}"
RAW_EXEC_DIR = "${exec_prefix}/libexec"
RAW_DATA_DIR = "${datarootdir}"
RAW_DATAROOT_DIR = "${prefix}/share"
RAW_BINDIR = "${exec_prefix}/bin"

EXEC_PREFIX = os.path.normpath (expandvars (RAW_EXEC_DIR))
DATADIR = os.path.normpath (expandvars (RAW_DATA_DIR))
BINDIR = os.path.normpath (expandvars (RAW_BINDIR))

haveUpstart = ("@HAVE_UPSTART_TRUE@" == "")
disableJournal = ("#" == "")

TEST_TMP_DIR = os.path.join (os.environ["HOME"], "tracker-tests")

TEST_MONITORED_TMP_DIR = TEST_TMP_DIR

if TEST_TMP_DIR.startswith('/tmp'):
	if os.environ.has_key('REAL_HOME'):
		TEST_MONITORED_TMP_DIR = os.path.join (os.environ["REAL_HOME"], "tracker-tests")
	else:
		print ("HOME is in the /tmp prefix - this will cause tests that rely " +
		       "on filesystem monitoring to fail as changes in that prefix are " +
		       "ignored.")



#!/usr/bin/env python
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
from common.utils import configuration as cfg
from common.utils.system import TrackerSystemAbstraction
import unittest2 as ut

from gi.repository import GLib

import shutil
import os
import warnings
from itertools import chain

MINER_TMP_DIR = cfg.TEST_MONITORED_TMP_DIR

def path (filename):
    return os.path.join (MINER_TMP_DIR, filename)

def uri (filename):
    return "file://" + os.path.join (MINER_TMP_DIR, filename)


DEFAULT_TEXT = "Some stupid content, to have a test file"

index_dirs = [os.path.join (MINER_TMP_DIR, "test-monitored")]
CONF_OPTIONS = {
    cfg.DCONF_MINER_SCHEMA: {
        'index-recursive-directories': GLib.Variant.new_strv(index_dirs),
        'index-single-directories': GLib.Variant.new_strv([]),
        'index-optical-discs': GLib.Variant.new_boolean(False),
        'index-removable-devices': GLib.Variant.new_boolean(False),
        'throttle': GLib.Variant.new_int32(5),
    }
}


class CommonTrackerMinerTest (ut.TestCase):

    def prepare_directories (self):
        #
        #     ~/test-monitored/
        #                     /file1.txt
        #                     /dir1/
        #                          /file2.txt
        #                          /dir2/
        #                               /file3.txt
        #
        #
        #     ~/test-no-monitored/
        #                        /file0.txt
        #

        monitored_files = [
            'test-monitored/file1.txt',
            'test-monitored/dir1/file2.txt',
            'test-monitored/dir1/dir2/file3.txt'
        ]

        unmonitored_files = [
            'test-no-monitored/file0.txt'
        ]

        def ensure_dir_exists(dirname):
            if not os.path.exists(dirname):
                os.makedirs(dirname)

        for tf in chain(monitored_files, unmonitored_files):
            testfile = path(tf)
            ensure_dir_exists(os.path.dirname(testfile))
            with open (testfile, 'w') as f:
                f.write (DEFAULT_TEXT)

        for tf in monitored_files:
            self.tracker.await_resource_inserted(
                'nfo:TextDocument', url=uri(tf))

    def setUp (self):
        for d in ['test-monitored', 'test-no-monitored']:
            dirname = path(d)
            if os.path.exists (dirname):
                shutil.rmtree(dirname)
            os.makedirs(dirname)

        self.system = TrackerSystemAbstraction ()

        self.system.tracker_miner_fs_testing_start (CONF_OPTIONS)
        self.tracker = self.system.store

        try:
            self.prepare_directories ()
            self.tracker.reset_graph_updates_tracking ()
        except Exception as e:
            self.tearDown ()
            raise

    def tearDown (self):
        self.system.tracker_miner_fs_testing_stop ()

    def assertResourceExists (self, urn):
        if self.tracker.ask ("ASK { <%s> a rdfs:Resource }" % urn) == False:
            self.fail ("Resource <%s> does not exist" % urn)

    def assertResourceMissing (self, urn):
        if self.tracker.ask ("ASK { <%s> a rdfs:Resource }" % urn) == True:
            self.fail ("Resource <%s> should not exist" % urn)



#!/usr/bin/python
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
"""
Send concurrent inserts and queries to the daemon to check the concurrency.
"""
import sys,os
import unittest
import time
import random
import commands
import signal
from gi.repository import GObject

from common.utils import configuration as cfg
import unittest2 as ut
#import unittest as ut
from common.utils.storetest import CommonTrackerStoreTest as CommonTrackerStoreTest

AMOUNT_OF_TEST_INSTANCES = 100
AMOUNT_OF_QUERIES = 10

class TestConcurrentQuery (CommonTrackerStoreTest):
    """
    Send a bunch of queries to the daemon asynchronously, to test the queue
    holding those queries
    """
    def setUp (self):
        self.main_loop = GObject.MainLoop ()
        
        self.mock_data_insert ()
        self.finish_counter = 0
        
    def mock_data_insert (self):
        query = "INSERT {\n"
        for i in range (0, AMOUNT_OF_TEST_INSTANCES):
            query += "<test-09:instance-%d> a nco:PersonContact ; nco:fullname 'moe %d'.\n" % (i, i)
        query += "}"
        self.tracker.update (query)
        
    def mock_data_delete (self):
        query = "DELETE {\n"
        for i in range (0, AMOUNT_OF_TEST_INSTANCES):
            query += "<test-09:instance-%d> a rdfs:Resource.\n" % (i)
        query += "}"
        self.tracker.update (query)

        query = "DELETE {\n"
        for i in range (0, AMOUNT_OF_QUERIES):
            query += "<test-09:picture-%d> a rdfs:Resource.\n" % (i)
        query += "}"
        self.tracker.update (query)


    def test_async_queries (self):
        QUERY = "SELECT ?u WHERE { ?u a nco:PersonContact. FILTER regex (?u, 'test-09:ins')}"
        UPDATE = "INSERT { <test-09:picture-%d> a nmm:Photo. }"
        for i in range (0, AMOUNT_OF_QUERIES):
            self.tracker.query(
                QUERY,
                result_handler=self.reply_cb,
                error_handler=self.error_handler)
            self.tracker.update(
                UPDATE % (i),
                result_handler=self.update_cb,
                error_handler=self.error_handler)

        # Safeguard of 50 seconds. The last reply should quit the loop
        GObject.timeout_add_seconds (60, self.timeout_cb)
        self.main_loop.run ()
        
    def reply_cb (self, obj, results, data):
        self.finish_counter += 1
        self.assertEquals (len (results), AMOUNT_OF_TEST_INSTANCES)
        if (self.finish_counter >= AMOUNT_OF_QUERIES):
            self.timeout_cb ()

    def update_cb (self, obj, results, data):
        self.assertTrue (True)

    def error_handler (self):
        print "ERROR in DBus call"
        self.assertTrue (False)

    def timeout_cb (self):
        self.mock_data_delete ()
        self.main_loop.quit ()
        return False

if __name__ == "__main__":
    ut.main ()



#!/usr/bin/python
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
from gi.repository import GLib

import os 

from common.utils.system import TrackerSystemAbstraction
from common.utils.helpers import StoreHelper
from common.utils import configuration as cfg
from common.utils.storetest import CommonTrackerStoreTest as CommonTrackerStoreTest
from common.utils.expectedFailure import expectedFailureBug, expectedFailureJournal
import unittest2 as ut


"""
Call backup, restore, force the journal replay and check the data is correct afterwards
"""
class BackupRestoreTest (CommonTrackerStoreTest):
        """
        Backup and restore to/from valid/invalid files
        """
        def setUp (self):
            self.TEST_INSTANCE = "test://backup-restore/1"
            self.BACKUP_FILE = "file://" + os.path.join (cfg.TEST_TMP_DIR, "tracker-backup-test-1")

            if (os.path.exists (self.BACKUP_FILE)):
                os.unlink (self.BACKUP_FILE)

        def __insert_test_instance (self):
            self.tracker.update ("INSERT { <%s> a nco:Contact; nco:fullname 'test-backup' } "
                                 % (self.TEST_INSTANCE))

        def __delete_test_instance (self):
            self.tracker.update ("DELETE { <%s> a rdfs:Resource } " % (self.TEST_INSTANCE))

        def __is_test_instance_there (self):
            result = self.tracker.query ("SELECT ?u WHERE { ?u a nco:Contact; nco:fullname 'test-backup'}")
            if (len (result) == 1 and len (result[0]) == 1 and result[0][0] == self.TEST_INSTANCE):
                return True
            return False

	def test_backup_01(self):
            """
            Inserted data is restored after backup
            
            1.Insert contact
            2.Take backup.
            3.Delete contact. (check it is not there)
            4.Restore the file.
            5.Check the contact is back there
            """
            
            self.__insert_test_instance ()
            instances_before = self.tracker.count_instances ("nco:Contact")
            
            self.tracker.backup (self.BACKUP_FILE)
            
            self.__delete_test_instance ()
            instances_now = self.tracker.count_instances ("nco:Contact")
            
            self.assertEquals (instances_before-1, instances_now)
            
            self.tracker.restore (self.BACKUP_FILE)

            instances_after = self.tracker.count_instances ("nco:Contact")

            self.assertEquals (instances_before, instances_after)
            self.assertTrue (self.__is_test_instance_there ())
            
            # Clean the DB for the next test
            self.__delete_test_instance ()


 	def test_backup_02 (self):
 	      """
              Data inserted after backup is lost in restore
              
 	      1.Take backup of db.
 	      2.Insert a contact.
 	      3.Restore the db.
 	      4.Search for the contact inserted. 
 	      """

              # Precondition: test backup contact shouldn't be there
              self.assertFalse (self.__is_test_instance_there ())

              self.tracker.backup (self.BACKUP_FILE)

              self.__insert_test_instance ()
              self.assertTrue (self.__is_test_instance_there ())

              self.tracker.restore (self.BACKUP_FILE)

              self.assertFalse (self.__is_test_instance_there ())


	
	def test_backup_03 (self):
	      """
              Restore from a random text file
	      """
              TEST_FILE = os.path.join (cfg.TEST_TMP_DIR, "trash_file")
              trashfile = open (TEST_FILE, "w")
              trashfile.write ("Here some useless text that obviously is NOT a backup")
              trashfile.close ()

              self.assertRaises(GLib.Error,
                                self.tracker.restore,
                                "file://" + TEST_FILE)
              os.unlink (TEST_FILE)

        def test_backup_04 (self):
              """
              Restore from a random binary file
              """
              TEST_FILE = os.path.join (cfg.TEST_TMP_DIR, "trash_file.dat")
              
              import struct
              trashfile = open (TEST_FILE, "wb")
              for n in range (0, 50):
                  data = struct.pack ('i', n)
                  trashfile.write (data)
              trashfile.close ()

              instances_before = self.tracker.count_instances ("nie:InformationElement")
              self.assertRaises(GLib.Error,
                                self.tracker.restore,
                                "file://" + TEST_FILE)

              os.unlink (TEST_FILE)

	def test_backup_05(self):
	      """
	      Take backup of db to a invalid path.
	      Expected: Backup should not be taken and tracker should behave normally.	
	      """
              self.assertRaises(GLib.Error,
                                self.tracker.backup,
                                "file://%s/this/is/a/non-existant/folder/backup" % (cfg.TEST_TMP_DIR))
              

        def test_backup_06 (self):
            """
            Try to restore an invalid path
            """
            self.assertRaises(GLib.Error,
                              self.tracker.restore,
                              "file://%s/this/is/a/non-existant/folder/backup" % (cfg.TEST_TMP_DIR))
		

	def test_backup_07(self):
	      """
              Restore after removing the DBs and journal
              
	      1.Insert a contact.
	      2.Take backup of db.
	      4.Delete the database
	      5.Restore the db.
	      6.Search for the contact inserted. 
              """
              self.__insert_test_instance ()
              instances_before = self.tracker.count_instances ("nco:Contact")
	      self.tracker.backup (self.BACKUP_FILE)

              self.system.tracker_store_stop_nicely ()
              self.system.tracker_store_remove_dbs ()
              self.system.tracker_store_remove_journal ()
              self.system.tracker_store_start ()
              
              instances_before_restore = self.tracker.count_instances ("nco:Contact")
              self.assertNotEqual (instances_before_restore, instances_before)
              
              self.tracker.restore (self.BACKUP_FILE)
	      self.assertTrue (self.__is_test_instance_there ())

              self.__delete_test_instance ()


	def test_backup_08 (self):
	      """
              Restore after corrupting DB
              
	      1.Insert a contact.
	      2.Take backup of db.
	      5.Restore the db.
	      6.Search for the contact inserted. 
              """
              self.__insert_test_instance ()
              instances_before = self.tracker.count_instances ("nco:Contact")
	      self.tracker.backup (self.BACKUP_FILE)

              self.system.tracker_store_stop_brutally ()
              self.system.tracker_store_corrupt_dbs ()
              self.system.tracker_store_remove_journal ()
              self.system.tracker_store_start ()
              
              instances_before_restore = self.tracker.count_instances ("nco:Contact")
              self.assertNotEqual (instances_before_restore, instances_before)
              
              self.tracker.restore (self.BACKUP_FILE)
	      self.assertTrue (self.__is_test_instance_there ())

              # DB to the original state
              self.__delete_test_instance ()
              
        def test_backup_11(self):
	      """
              Backup ignores the file extension
              
	      1.Insert a contact.
	      2.Take backup of db in .jpg format.
	      3.Restore the db.
	      4.Search for the contact inserted. 
	      """
              BACKUP_JPG_EXT = "file://%s/tracker-test-backup.jpg" % (cfg.TEST_TMP_DIR)
              
              self.__insert_test_instance ()

              instances_before = self.tracker.count_instances ("nco:Contact")

	      self.tracker.backup (BACKUP_JPG_EXT)

              self.__delete_test_instance ()
              instances_now = self.tracker.count_instances ("nco:Contact")
              self.assertEquals (instances_before, instances_now+1)

              self.tracker.restore (BACKUP_JPG_EXT)
              instances_after = self.tracker.count_instances ("nco:Contact")
              self.assertEquals (instances_before, instances_after)

              # Restore the DB to the original state
              self.__delete_test_instance ()



class JournalReplayTest (CommonTrackerStoreTest):
        """
        Force journal replaying and check that the DB is correct aftewards
        """
	@expectedFailureJournal()
 	def test_journal_01 (self) :
            """
            Journal replaying when the DB is corrupted
            
            Insert few data (to have more than the pre-defined instances)
            Check instances of different classes
            Kill the store (SIGKILL)
            Replace the DB with a random file
            Start the store
            Check instances of different classes
            """
            self.tracker.update ("INSERT { <test://journal-replay/01> a nco:Contact. }")
            
            emails = self.tracker.count_instances ("nmo:Email")
            ie = self.tracker.count_instances ("nie:InformationElement")
            contacts = self.tracker.count_instances ("nco:Contact")

            self.system.tracker_store_stop_brutally ()
            self.system.tracker_store_corrupt_dbs ()
            self.system.tracker_store_start ()

            emails_now = self.tracker.count_instances ("nmo:Email")
            ie_now = self.tracker.count_instances ("nie:InformationElement")
            contacts_now = self.tracker.count_instances ("nco:Contact")

            self.assertEquals (emails, emails_now)
            self.assertEquals (ie, ie_now)
            self.assertEquals (contacts, contacts_now)

            self.tracker.update ("DELETE { <test://journal-replay/01> a rdfs:Resource. }")

	@expectedFailureJournal()
	def test_journal_02 (self) :
            """
            Journal replaying when the DB disappears
            
            Insert few data (to have more than the pre-defined instances)
            Check instances of different classes
            KILL the store
            Force a journal replay (DB missing, .meta.isrunning exists, journal OK)
            Start the store
            Check instances of different classes
            """
            self.tracker.update ("INSERT { <test://journal-replay/02> a nco:Contact. }")
            
            emails = self.tracker.count_instances ("nmo:Email")
            ie = self.tracker.count_instances ("nie:InformationElement")
            contacts = self.tracker.count_instances ("nco:Contact")

            self.system.tracker_store_stop_brutally ()
            self.system.tracker_store_prepare_journal_replay ()
            self.system.tracker_store_start ()

            emails_now = self.tracker.count_instances ("nmo:Email")
            ie_now = self.tracker.count_instances ("nie:InformationElement")
            contacts_now = self.tracker.count_instances ("nco:Contact")

            self.assertEquals (emails, emails_now)
            self.assertEquals (ie, ie_now)
            self.assertEquals (contacts, contacts_now)

            self.tracker.update ("DELETE { <test://journal-replay/02> a rdfs:Resource. }")

if __name__ == "__main__":
    ut.main()                      



